package edu.lawrence.androidgrades

import android.annotation.SuppressLint
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AccountCircle
import androidx.compose.material.icons.filled.CheckCircle
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material3.BottomAppBar
import androidx.compose.material3.Button
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.semantics.Role.Companion.DropdownList
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.room.Room
import edu.lawrence.androidgrades.ui.theme.AndroidGradesTheme
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        val vm = ViewModelProvider(
            this,
            ViewModelProvider.AndroidViewModelFactory.getInstance(application)
        ).get(GradesModel::class.java)

        setContent {
            AndroidGradesTheme {
                GradesApp(vm)
            }
        }
    }
}

@Composable
fun GradesApp(vm : GradesModel) {
    val navController = rememberNavController()
    Scaffold (
        bottomBar = {
            BottomAppBar(
                actions = {
                    IconButton(onClick = { navController.navigate("Students")}) {
                        Icon(Icons.Filled.AccountCircle,contentDescription="Students")
                    }
                    IconButton(onClick = { navController.navigate("Quizzes")}) {
                        Icon(Icons.Filled.Menu,contentDescription="Quizzes")
                    }
                    IconButton(onClick = { navController.navigate("Grades")}) {
                        Icon(Icons.Filled.CheckCircle,contentDescription="Grades")
                    }
                }
            )
        }
    ) { innerPadding ->
        val modifier = Modifier
            .fillMaxSize()
            .padding(innerPadding)
        NavHost(navController,startDestination="Students") {
            composable(route="Students") {
                StudentScreen(vm,modifier=modifier)
            }
            composable(route="Quizzes") {
                QuizScreen(vm,modifier = modifier)
            }
            composable(route="Grades") {
                vm.initGrade()
                GradeScreen(vm,modifier = modifier)
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun StudentScreen(vm : GradesModel,modifier: Modifier) {
    val students = remember { vm.students }
    val newStudentLastName = remember { mutableStateOf("") }
    val newStudentFirstName = remember { mutableStateOf("") }
    val focusManager = LocalFocusManager.current

    Column(verticalArrangement = Arrangement.SpaceBetween, modifier = modifier.padding(20.dp)) {
        if (students.value == null) {
            Text("Loading...")
        } else {
            Column(verticalArrangement = Arrangement.Top, modifier = Modifier.fillMaxWidth()) {
                Spacer(modifier = Modifier.height(20.dp))
                students.value!!.forEach { student ->
                    Text("${student.firstName} ${student.lastName}")
                }
            }
        }
        Column(verticalArrangement = Arrangement.Bottom, modifier = Modifier.fillMaxWidth()) {
            Row(
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.Bottom,
                modifier = Modifier.fillMaxWidth()
            ) {
                Text("First Name")
                Spacer(modifier = Modifier.width(10.dp))
                TextField(value = newStudentFirstName.value,
                    onValueChange = { newValue: String -> newStudentFirstName.value = newValue },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                    keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
                )
            }
            Spacer(modifier = Modifier.height(10.dp))
            Row(
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.Bottom,
                modifier = Modifier.fillMaxWidth()
            ) {
                Text("Last Name")
                Spacer(modifier = Modifier.width(10.dp))
                TextField(value = newStudentLastName.value,
                    onValueChange = { newValue: String -> newStudentLastName.value = newValue },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                    keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
                )
            }
            Spacer(modifier = Modifier.height(10.dp))
            Button(onClick = {
                val newStudent = Student(
                    firstName = newStudentFirstName.value,
                    lastName = newStudentLastName.value
                )
                vm.addStudent(newStudent)
            }) {
                Text("Save")
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun QuizScreen(vm : GradesModel,modifier: Modifier) {
    val quizzes = remember { vm.quizzes }
    val newQuizName = remember { mutableStateOf("") }
    val newQuizPoints = remember { mutableStateOf("100") }
    val focusManager = LocalFocusManager.current

    Column(verticalArrangement = Arrangement.SpaceBetween, modifier = modifier.padding(20.dp)) {
        if (quizzes.value == null) {
            Text("Loading...")
        } else {
            Column(verticalArrangement = Arrangement.Top, modifier = Modifier.fillMaxWidth()) {
                Spacer(modifier = Modifier.height(20.dp))
                quizzes.value!!.forEach { quiz ->
                    Text("${quiz.name} : ${quiz.maxPoints} points")
                }
            }
        }
        Column(verticalArrangement = Arrangement.Bottom, modifier = Modifier.fillMaxWidth()) {
            Row(
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.Bottom,
                modifier = Modifier.fillMaxWidth()
            ) {
                Text("Quiz")
                Spacer(modifier = Modifier.width(10.dp))
                TextField(value = newQuizName.value,
                    onValueChange = { newValue: String -> newQuizName.value = newValue },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                    keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
                )
            }
            Spacer(modifier = Modifier.height(10.dp))
            Row(
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.Bottom,
                modifier = Modifier.fillMaxWidth()
            ) {
                Text("Points")
                Spacer(modifier = Modifier.width(10.dp))
                TextField(value = newQuizPoints.value,
                    onValueChange = { newValue: String -> newQuizPoints.value = newValue },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                    keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
               )
            }
            Spacer(modifier = Modifier.height(10.dp))
            Button(onClick = {
                val newQuiz =
                    Quiz(name = newQuizName.value, maxPoints = newQuizPoints.value.toInt())
                vm.addQuiz(newQuiz)
            }) {
                Text("Save")
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun GradeScreen(vm : GradesModel,modifier: Modifier) {
    val students = remember { vm.students }
    val quizzes = remember { vm.quizzes }
    val focusManager = LocalFocusManager.current
    val studentsExpanded = remember { mutableStateOf(false) }
    val selectedStudent = remember { vm.firstStudent }
    val quizzesExpanded = remember { mutableStateOf(false) }
    val selectedQuiz = remember { vm.firstQuiz }
    val grade = remember { vm.grade }

    val score = remember { mutableStateOf("") }
    Column(verticalArrangement = Arrangement.SpaceBetween, modifier = modifier.padding(20.dp)) {

        Column(verticalArrangement = Arrangement.Top, modifier = Modifier.fillMaxWidth()) {
            Spacer(modifier = Modifier.height(20.dp))
            if (students.value == null) {
                Text("Loading students...")
            } else if(students.value!!.isEmpty()) {
                Text("No students to display")
            } else {
                ExposedDropdownMenuBox(
                    expanded = studentsExpanded.value,
                    onExpandedChange = { studentsExpanded.value = it },
                ) {
                    TextField(
                        value = "${selectedStudent.value?.firstName} ${selectedStudent.value?.lastName}",
                        onValueChange = {},
                        readOnly = true,
                        trailingIcon = {
                            ExposedDropdownMenuDefaults.TrailingIcon(expanded = studentsExpanded.value)
                        },
                        placeholder = {
                            Text(text = "Please select a student")
                        },
                        colors = ExposedDropdownMenuDefaults.textFieldColors(),
                        modifier = Modifier.menuAnchor()
                    )
                    ExposedDropdownMenu(
                        expanded = studentsExpanded.value,
                        onDismissRequest = { studentsExpanded.value = false },
                    ) {
                        students.value!!.forEach { student ->
                            DropdownMenuItem(
                                text = {
                                    Text(
                                        "${student.firstName} ${student.lastName}",
                                        style = MaterialTheme.typography.bodyLarge
                                    )
                                },
                                onClick = {
                                    selectedStudent.value = student
                                    studentsExpanded.value = false
                                    if(selectedStudent.value != null && selectedQuiz.value != null)
                                        vm.getGrade(selectedStudent.value!!,selectedQuiz.value!!)
                                },
                                contentPadding = ExposedDropdownMenuDefaults.ItemContentPadding,
                            )
                        }
                    }
                }

            }
            Spacer(modifier = Modifier.height(10.dp))
            if (quizzes.value == null) {
                Text("Loading quizzes...")
            } else if(quizzes.value!!.isEmpty()) {
                Text("No quizzes to display")
            } else {
                ExposedDropdownMenuBox(
                    expanded = quizzesExpanded.value,
                    onExpandedChange = { quizzesExpanded.value = it },
                ) {
                    TextField(
                        value = selectedQuiz.value?.name?:"unknown",
                        onValueChange = {},
                        readOnly = true,
                        trailingIcon = {
                            ExposedDropdownMenuDefaults.TrailingIcon(expanded = quizzesExpanded.value)
                        },
                        placeholder = {
                            Text(text = "Please select a quiz")
                        },
                        colors = ExposedDropdownMenuDefaults.textFieldColors(),
                        modifier = Modifier.menuAnchor()
                    )
                    ExposedDropdownMenu(
                        expanded = quizzesExpanded.value,
                        onDismissRequest = { quizzesExpanded.value = false },
                    ) {
                        quizzes.value!!.forEach { quiz ->
                            DropdownMenuItem(
                                text = {
                                    Text(
                                        "${quiz.name}",
                                        style = MaterialTheme.typography.bodyLarge
                                    )
                                },
                                onClick = {
                                    selectedQuiz.value = quiz
                                    quizzesExpanded.value = false
                                    if(selectedStudent.value != null && selectedQuiz.value != null)
                                        vm.getGrade(selectedStudent.value!!,selectedQuiz.value!!)
                                },
                                contentPadding = ExposedDropdownMenuDefaults.ItemContentPadding,
                            )
                        }
                    }
                }
            }
        }
        Text("Existing score: ${grade.value?.grade?.toString()?:"not entered"}")
        Column(verticalArrangement = Arrangement.Bottom, modifier = Modifier.fillMaxWidth()) {
            Row(
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.Bottom,
                modifier = Modifier.fillMaxWidth()
            ) {
                Text("Score")
                Spacer(modifier = Modifier.width(10.dp))
                TextField(
                    value = score.value,
                    onValueChange = { newValue: String -> score.value = newValue },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                    keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
                )
            }
            Spacer(modifier = Modifier.height(10.dp))
            Button(onClick = {
                if (selectedStudent.value != null && selectedQuiz.value != null && score.value.toIntOrNull() != null) {
                    val newGrade = Grade(
                        student = selectedStudent.value!!.id,
                        quiz = selectedQuiz.value!!.id,
                        grade = score.value.toInt()
                    )
                    vm.addGrade(newGrade)
                }
            }) {
                Text("Save")
            }
        }
    }
}